﻿Public Class Main

#Region " Declarations "
    Private Const _WRONG_NUM_ARGUMENTS As Integer = 1
    Private Const _UNKNOWN_EXCEPTION As Integer = 2

    Private Const _NUM_ARGUMENTS As Integer = 2
    Private Const _EMPTY_ARRAY_SIZE As Integer = 128
#End Region

#Region " Public Methods "
    Public Shared Sub Main(ByVal args() As String)
        'Validate correct number of arguments
        If args.Length <> _NUM_ARGUMENTS Then
            _DisplayUsage(_WRONG_NUM_ARGUMENTS, "Invalid number of arguments!")
            Exit Sub
        End If

        Try
            Dim infile As New IO.FileStream(args(0), IO.FileMode.Open)
            Dim ininfo As New IO.FileInfo(args(0))

            Dim inputData(Convert.ToInt32(ininfo.Length - 1)) As Byte
            infile.Read(inputData, 0, Convert.ToInt32(ininfo.Length))
            infile.Close()

            Dim outfile As New IO.FileStream(args(1) & ".8xv", IO.FileMode.Create)

            Dim checksum As UInt16 = &HD + &H15
            Dim emptyArray(_EMPTY_ARRAY_SIZE) As Byte
            Dim nameArray(8) As Byte
            For i As Integer = 0 To args(1).Length - 1
                nameArray(i) = Text.ASCIIEncoding.ASCII.GetBytes(args(1).ToUpper()(i))(0)
                checksum = Convert.ToUInt16((checksum + nameArray(i)) And &HFFFF)
            Next
            Dim dataSize As UInt16 = Convert.ToUInt16(ininfo.Length + 13 + 2 + 2 + 2)
            Dim shortArray() As Byte
            'For i As Integer = 0 To _EMPTY_ARRAY_SIZE - 1
            '    emptyArray(i) = 0
            'Next
            outfile.Write(Text.ASCIIEncoding.ASCII.GetBytes("**TI83F*"), 0, 8)
            outfile.Write(New Byte() {&H1A, &HA, &H0}, 0, 3)
            outfile.Write(emptyArray, 0, 42)
            outfile.Write(BitConverter.GetBytes(dataSize), 0, 2)
            outfile.Write(New Byte() {&HD, &H0}, 0, 2)
            shortArray = BitConverter.GetBytes(Convert.ToUInt16(ininfo.Length + 2))
            checksum = Convert.ToUInt16((checksum + shortArray(0)) And &HFFFF)
            checksum = Convert.ToUInt16((checksum + shortArray(1)) And &HFFFF)
            outfile.Write(shortArray, 0, 2)
            outfile.WriteByte(&H15)
            outfile.Write(nameArray, 0, 8)
            outfile.WriteByte(0)
            'checksum += Convert.ToUInt16(&H80)
            outfile.WriteByte(0) '&H80)
            shortArray = BitConverter.GetBytes(Convert.ToUInt16(ininfo.Length + 2))
            checksum = Convert.ToUInt16((checksum + shortArray(0)) And &HFFFF)
            checksum = Convert.ToUInt16((checksum + shortArray(1)) And &HFFFF)
            outfile.Write(shortArray, 0, 2)
            shortArray = BitConverter.GetBytes(Convert.ToUInt16(ininfo.Length))
            checksum = Convert.ToUInt16((checksum + shortArray(0)) And &HFFFF)
            checksum = Convert.ToUInt16((checksum + shortArray(1)) And &HFFFF)
            outfile.Write(shortArray, 0, 2)
            Dim temp As Integer
            For i As Integer = 0 To inputData.Length - 1
                temp = checksum
                temp += inputData(i)
                checksum = Convert.ToUInt16(temp And &HFFFF)
            Next
            outfile.Write(inputData, 0, inputData.Length)
            outfile.Write(BitConverter.GetBytes(checksum), 0, 2)

            outfile.Close()
        Catch ex As Exception
            _DisplayUsage(_UNKNOWN_EXCEPTION, ex.Message)
            Exit Sub
        End Try
    End Sub
#End Region

#Region " Private Methods "
    Private Shared Sub _DisplayUsage(ByVal exitCode As Integer, ByVal msg As String)
        Console.WriteLine("To8XV " & My.Application.Info.Version.ToString(3))
        Console.WriteLine("------------------------------------------------------")
        Console.WriteLine("Usage: To8XV <BIN file> <8XV file name (no extension)>")
        Console.WriteLine("Converts a binary image to an 8XV AppVar variable.")
        Console.WriteLine()
        Console.WriteLine("Examples: To8XV data.bin MYTEST")
        Console.WriteLine("  Creates an AppVar called MYTEST.8XV with oncalc name")
        Console.WriteLine("  ""MYTEST"" and with same size as data.bin.")
        Console.WriteLine()
        Console.Write("ERROR: " & msg)

        Environment.ExitCode = exitCode
    End Sub
#End Region

End Class
